==> tencent-website-beta-1.2.16.tgz: package/serverless.js <==
const COS = require('cos-nodejs-sdk-v5')
const TencentLogin = require('tencent-login')
const path = require('path')
const util = require('util')
const fs = require('fs')
const exec = util.promisify(require('child_process').exec)
const { Component, utils } = require('@serverless/core')
const { configureBucketForHosting } = require('./utils')
const tencentcloud = require('tencentcloud-sdk-nodejs')
const ClientProfile = require('tencentcloud-sdk-nodejs/tencentcloud/common/profile/client_profile.js')
const HttpProfile = require('tencentcloud-sdk-nodejs/tencentcloud/common/profile/http_profile.js')
const AbstractModel = require('tencentcloud-sdk-nodejs/tencentcloud/common/abstract_model')
const AbstractClient = require('tencentcloud-sdk-nodejs/tencentcloud/common/abstract_client')

class GetUserAppIdResponse extends AbstractModel {
  constructor() {
    super()

    this.RequestId = null
  }

  deserialize(params) {
    if (!params) {
      return
    }
    this.AppId = 'RequestId' in params ? params.AppId : null
    this.RequestId = 'RequestId' in params ? params.RequestId : null
  }
}

class AppidClient extends AbstractClient {
  constructor(credential, region, profile) {
    super('cam.tencentcloudapi.com', '2019-01-16', credential, region, profile)
  }

  GetUserAppId(req, cb) {
    const resp = new GetUserAppIdResponse()
    this.request('GetUserAppId', req, resp, cb)
  }
}

class Website extends Component {
  confirmEnding(sourceStr, targetStr) {
    // judge the sourceStr end with targetStr
    targetStr = targetStr.toString()
    const start = sourceStr.length - targetStr.length
    const arr = sourceStr.substr(start, targetStr.length)
    if (arr == targetStr) {
      return true
    }
    return false
  }

  getAppid(credentials) {
    const secret_id = credentials.SecretId
    const secret_key = credentials.SecretKey
    const cred = credentials.token
      ? new tencentcloud.common.Credential(secret_id, secret_key, credentials.token)
      : new tencentcloud.common.Credential(secret_id, secret_key)
    const httpProfile = new HttpProfile()
    httpProfile.reqTimeout = 30
    const clientProfile = new ClientProfile('HmacSHA256', httpProfile)
    const cam = new AppidClient(cred, 'ap-guangzhou', clientProfile)
    const req = new GetUserAppIdResponse()
    const body = {}
    req.from_json_string(JSON.stringify(body))
    const handler = util.promisify(cam.GetUserAppId.bind(cam))
    try {
      return handler(req)
    } catch (e) {
      throw 'Get Appid failed! '
    }
  }

  async doLogin() {
    const login = new TencentLogin()
    const tencent_credentials = await login.login()
    if (tencent_credentials) {
      tencent_credentials.timestamp = Date.now() / 1000
      try {
        const tencent = {
          SecretId: tencent_credentials.secret_id,
          SecretKey: tencent_credentials.secret_key,
          AppId: tencent_credentials.appid,
          token: tencent_credentials.token,
          expired: tencent_credentials.expired,
          signature: tencent_credentials.signature,
          uuid: tencent_credentials.uuid,
          timestamp: tencent_credentials.timestamp
        }
        await fs.writeFileSync('./.env_temp', JSON.stringify(tencent))
        return tencent
      } catch (e) {
        throw 'Error getting temporary key: ' + e
      }
    }
  }

  async sleep(ms) {
    return new Promise((resolve) => {
      setTimeout(resolve, ms)
    })
  }

  async getTempKey(temp) {
    const that = this

    if (temp) {
      while (true) {
        try {
          const tencent_credentials_read = JSON.parse(await fs.readFileSync('./.env_temp', 'utf8'))
          if (
            Date.now() / 1000 - tencent_credentials_read.timestamp <= 5 &&
            tencent_credentials_read.AppId
          ) {
            return tencent_credentials_read
          }
          await that.sleep(1000)
        } catch (e) {
          await that.sleep(1000)
        }
      }
    }

    try {
      const data = await fs.readFileSync('./.env_temp', 'utf8')
      try {
        const tencent = {}
        const tencent_credentials_read = JSON.parse(data)
        if (
          Date.now() / 1000 - tencent_credentials_read.timestamp <= 6000 &&
          tencent_credentials_read.AppId
        ) {
          return tencent_credentials_read
        }
        const login = new TencentLogin()
        const tencent_credentials_flush = await login.flush(
          tencent_credentials_read.uuid,
          tencent_credentials_read.expired,
          tencent_credentials_read.signature,
          tencent_credentials_read.AppId
        )
        if (tencent_credentials_flush) {
          tencent.SecretId = tencent_credentials_flush.secret_id
          tencent.SecretKey = tencent_credentials_flush.secret_key
          tencent.AppId = tencent_credentials_flush.appid
          tencent.token = tencent_credentials_flush.token
          tencent.expired = tencent_credentials_flush.expired
          tencent.signature = tencent_credentials_flush.signature
          tencent.uuid = tencent_credentials_read.uuid
          tencent.timestamp = Date.now() / 1000
          await fs.writeFileSync('./.env_temp', JSON.stringify(tencent))
          return tencent
        }
        return await that.doLogin()
      } catch (e) {
        return await that.doLogin()
      }
    } catch (e) {
      return await that.doLogin()
    }
  }

  async default(inputs = {}) {
    // login
    const temp = this.context.instance.state.status
    this.context.instance.state.status = true
    this.context.status('Deploying')
    this.context.debug(`Starting Website Component.`)
    let { tencent } = this.context.credentials
    if (!tencent) {
      tencent = await this.getTempKey(temp)
      this.context.credentials.tencent = tencent
    }

    const option = {
      region: inputs.region || 'ap-guangzhou',
      timestamp: this.context.credentials.tencent.timestamp || null,
      token: this.context.credentials.tencent.token || null
    }
    if (!this.context.credentials.tencent.AppId) {
      const appId = await this.getAppid(tencent)
      this.context.credentials.tencent.AppId = appId.AppId
    }
    // Default to current working directory
    const appId = await this.getAppid(this.context.credentials.tencent)
    this.context.credentials.tencent.AppId = appId.AppId
    inputs.code = inputs.code || {}
    inputs.code.root = inputs.code.root ? path.resolve(inputs.code.root) : process.cwd()
    inputs.code.index = inputs.code.index || 'index.html'
    inputs.code.error = inputs.code.error || 'error.html'
    if (inputs.code.src) {
      inputs.code.src = path.join(inputs.code.root, inputs.code.src)
    }
    inputs.region = inputs.region || 'ap-guangzhou'
    inputs.bucketName = this.state.bucketName || inputs.bucketName || this.context.resourceId()
    if (!this.confirmEnding(inputs.bucketName, this.context.credentials.tencent.AppId)) {
      inputs.bucketName = inputs.bucketName + '-' + this.context.credentials.tencent.AppId
    }

    this.context.status(`Preparing Tencent COS Bucket`)
    this.context.debug(`Preparing website Tencent COS bucket ${inputs.bucketName}.`)

    const websiteBucket = await this.load('@tencent-serverless/tencent-cos-beta', 'websiteBucket')
    await websiteBucket({
      bucket: inputs.bucketName,
      region: inputs.region
    })
    this.state.bucketName = inputs.bucketName
    await this.save()

    let cos
    if (!option.token) {
      cos = new COS({
        SecretId: this.context.credentials.tencent.SecretId,
        SecretKey: this.context.credentials.tencent.SecretKey,
        UserAgent: 'ServerlessComponent'
      })
    } else {
      const tencentConf = this.context.credentials.tencent
      cos = new COS({
        getAuthorization: function(option, callback) {
          callback({
            TmpSecretId: tencentConf.SecretId,
            TmpSecretKey: tencentConf.SecretKey,
            UserAgent: 'ServerlessComponent',
            XCosSecurityToken: tencentConf.token,
            ExpiredTime: tencentConf.timestamp
          })
        }
      })
    }

    inputs.protocol = inputs.protocol || 'http'
    this.context.debug(`Configuring bucket ${inputs.bucketName} for website hosting.`)
    await configureBucketForHosting(
      cos,
      inputs.bucketName,
      this.context.credentials.tencent.AppId,
      inputs.region,
      inputs.code.index,
      inputs.code.error,
      inputs.cors || null,
      inputs.protocol
    )

    // Build environment variables
    if (inputs.env && Object.keys(inputs.env).length && inputs.code.root) {
      this.context.status(`Bundling environment variables`)
      this.context.debug(`Bundling website environment variables.`)
      let script = 'window.env = {};\n'
      inputs.env = inputs.env || {}
      for (const e in inputs.env) {
				// eslint-disable-line
				script += `window.env.${e} = ${JSON.stringify(inputs.env[e])};\n` // eslint-disable-line
      }
      const envFilePath = path.join(inputs.code.root, 'env.js')
      await utils.writeFile(envFilePath, script)
      this.context.debug(`Website env written to file ${envFilePath}.`)
    }

    // If a hook is provided, build the website
    if (inputs.code.hook) {
      this.context.status('Building assets')
      this.context.debug(`Running ${inputs.code.hook} in ${inputs.code.root}.`)

      const options = { cwd: inputs.code.root }
      try {
        await exec(inputs.code.hook, options)
      } catch (err) {
				console.error(err.stderr) // eslint-disable-line
        throw new Error(
          `Failed building website via "${inputs.code.hook}" due to the following error: "${err.stderr}"`
        )
      }
    }

    this.context.status('Uploading')

    const dirToUploadPath = inputs.code.src || inputs.code.root

    this.context.debug(
      `Uploading website files from ${dirToUploadPath} to bucket ${inputs.bucketName}.`
    )

    if (fs.lstatSync(dirToUploadPath).isDirectory()) {
      await websiteBucket.upload({ dir: dirToUploadPath })
    } else {
      await websiteBucket.upload({ file: dirToUploadPath })
    }

    const cosOriginAdd = `${inputs.bucketName}.cos-website.${inputs.region}.myqcloud.com`

    // add user domain
    if (inputs.hosts && inputs.hosts.length > 0) {
      let tencentCdn
      let tencentCdnOutput
      let cdnInputs
      this.state.host = new Array()
      this.state.hostName = new Array()
      for (let i = 0; i < inputs.hosts.length; i++) {
        cdnInputs = inputs.hosts[i]
        cdnInputs.hostType = 'cos'
        cdnInputs.serviceType = 'web'
        cdnInputs.fwdHost = cosOriginAdd
        cdnInputs.origin = cosOriginAdd
        tencentCdn = await this.load(
          '@tencent-serverless/tencent-cdn-beta',
          inputs.hosts[i].host.replace('.', '_')
        )
        tencentCdnOutput = await tencentCdn(cdnInputs)
        const protocol = tencentCdnOutput.https ? 'https' : 'http'
        this.state.host.push(
          protocol + '://' + tencentCdnOutput.host + ' (CNAME: ' + tencentCdnOutput.cname + '）'
        )
        this.state.hostName.push(inputs.hosts[i].host.replace('.', '_'))
      }
    }

    this.state.bucketName = inputs.bucketName
    this.state.region = inputs.region
    this.state.url = `${inputs.protocol}://${cosOriginAdd}`
    await this.save()

    const outputs = {
      url: this.state.url,
      env: inputs.env || {}
    }

    if (this.state.host) {
      outputs.host = this.state.host
    }

    this.context.debug(`Website deployed successfully to URL: ${this.state.url}.`)

    return outputs
  }

  /**
   * Remove
   */

  async remove() {
    this.context.status(`Removing`)

    this.context.debug(`Starting Website Removal.`)

    this.context.debug(`Removing Website bucket.`)
    const websiteBucket = await this.load('@tencent-serverless/tencent-cos-beta', 'websiteBucket')
    await websiteBucket.remove()

    if (this.state.hostName && this.state.hostName.length > 0) {
      let tencentCdn
      for (let i = 0; i < this.state.hostName.length; i++) {
        tencentCdn = await this.load('@tencent-serverless/tencent-cdn-beta', this.state.hostName[i])
        await tencentCdn.remove()
      }
    }

    this.state = {}
    await this.save()

    this.context.debug(`Finished Website Removal.`)
    return {}
  }
}

module.exports = Website

==> tencent-website-beta-1.2.18.tgz: package/serverless.js <==
const COS = require('cos-nodejs-sdk-v5')
const path = require('path')
const util = require('util')
const fs = require('fs')
const exec = util.promisify(require('child_process').exec)
const { Component, utils } = require('@serverless/core')
const { configureBucketForHosting } = require('./utils')
// const tencentAuth = require('serverless-tencent-auth-tool')
const tencentAuth = require('/Users/dfounderliu/Desktop/component/experience/serverless-tencent-auth-tool')

class Website extends Component {
  confirmEnding(sourceStr, targetStr) {
    // judge the sourceStr end with targetStr
    targetStr = targetStr.toString()
    const start = sourceStr.length - targetStr.length
    const arr = sourceStr.substr(start, targetStr.length)
    if (arr == targetStr) {
      return true
    }
    return false
  }

  async default(inputs = {}) {
    // login
    const auth = new tencentAuth()
    this.context.credentials.tencent = await auth.doAuth(this.context.credentials.tencent)
    const { tencent } = this.context.credentials

    const option = {
      region: inputs.region || 'ap-guangzhou',
      timestamp: this.context.credentials.tencent.timestamp || null,
      token: this.context.credentials.tencent.token || null
    }
    if (!this.context.credentials.tencent.AppId) {
      const appId = await this.getAppid(tencent)
      this.context.credentials.tencent.AppId = appId.AppId
    }
    // Default to current working directory
    inputs.code = inputs.code || {}
    inputs.code.root = inputs.code.root ? path.resolve(inputs.code.root) : process.cwd()
    inputs.code.index = inputs.code.index || 'index.html'
    inputs.code.error = inputs.code.error || 'error.html'
    if (inputs.code.src) {
      inputs.code.src = path.join(inputs.code.root, inputs.code.src)
    }
    inputs.region = inputs.region || 'ap-guangzhou'
    inputs.bucketName = this.state.bucketName || inputs.bucketName || this.context.resourceId()
    if (!this.confirmEnding(inputs.bucketName, this.context.credentials.tencent.AppId)) {
      inputs.bucketName = inputs.bucketName + '-' + this.context.credentials.tencent.AppId
    }

    this.context.status(`Preparing Tencent COS Bucket`)
    this.context.debug(`Preparing website Tencent COS bucket ${inputs.bucketName}.`)

    const websiteBucket = await this.load('@serverless/tencent-cos', 'websiteBucket')
    await websiteBucket({
      bucket: inputs.bucketName,
      region: inputs.region
    })
    this.state.bucketName = inputs.bucketName
    await this.save()

    let cos
    if (!option.token) {
      cos = new COS({
        SecretId: this.context.credentials.tencent.SecretId,
        SecretKey: this.context.credentials.tencent.SecretKey,
        UserAgent: 'ServerlessComponent'
      })
    } else {
      const tencentConf = this.context.credentials.tencent
      cos = new COS({
        getAuthorization: function(option, callback) {
          callback({
            TmpSecretId: tencentConf.SecretId,
            TmpSecretKey: tencentConf.SecretKey,
            UserAgent: 'ServerlessComponent',
            XCosSecurityToken: tencentConf.token,
            ExpiredTime: tencentConf.timestamp
          })
        }
      })
    }

    inputs.protocol = inputs.protocol || 'http'
    this.context.debug(`Configuring bucket ${inputs.bucketName} for website hosting.`)
    await configureBucketForHosting(
      cos,
      inputs.bucketName,
      this.context.credentials.tencent.AppId,
      inputs.region,
      inputs.code.index,
      inputs.code.error,
      inputs.cors || null,
      inputs.protocol
    )

    // Build environment variables
    if (inputs.env && Object.keys(inputs.env).length && inputs.code.root) {
      this.context.status(`Bundling environment variables`)
      this.context.debug(`Bundling website environment variables.`)
      let script = 'window.env = {};\n'
      inputs.env = inputs.env || {}
      for (const e in inputs.env) {
        // eslint-disable-line
        script += `window.env.${e} = ${JSON.stringify(inputs.env[e])};\n` // eslint-disable-line
      }
      const envFilePath = path.join(inputs.code.root, 'env.js')
      await utils.writeFile(envFilePath, script)
      this.context.debug(`Website env written to file ${envFilePath}.`)
    }

    // If a hook is provided, build the website
    if (inputs.code.hook) {
      this.context.status('Building assets')
      this.context.debug(`Running ${inputs.code.hook} in ${inputs.code.root}.`)

      const options = { cwd: inputs.code.root }
      try {
        await exec(inputs.code.hook, options)
      } catch (err) {
        console.error(err.stderr) // eslint-disable-line
        throw new Error(
          `Failed building website via "${inputs.code.hook}" due to the following error: "${err.stderr}"`
        )
      }
    }

    this.context.status('Uploading')

    const dirToUploadPath = inputs.code.src || inputs.code.root

    this.context.debug(
      `Uploading website files from ${dirToUploadPath} to bucket ${inputs.bucketName}.`
    )

    if (fs.lstatSync(dirToUploadPath).isDirectory()) {
      await websiteBucket.upload({ dir: dirToUploadPath })
    } else {
      await websiteBucket.upload({ file: dirToUploadPath })
    }

    const cosOriginAdd = `${inputs.bucketName}.cos-website.${inputs.region}.myqcloud.com`

    // add user domain
    if (inputs.hosts && inputs.hosts.length > 0) {
      let tencentCdn
      let tencentCdnOutput
      let cdnInputs
      this.state.host = new Array()
      this.state.hostName = new Array()
      for (let i = 0; i < inputs.hosts.length; i++) {
        cdnInputs = inputs.hosts[i]
        cdnInputs.hostType = 'cos'
        cdnInputs.serviceType = 'web'
        cdnInputs.fwdHost = cosOriginAdd
        cdnInputs.origin = cosOriginAdd
        tencentCdn = await this.load(
          '@serverless/tencent-cdn',
          inputs.hosts[i].host.replace('.', '_')
        )
        tencentCdnOutput = await tencentCdn(cdnInputs)
        const protocol = tencentCdnOutput.https ? 'https' : 'http'
        this.state.host.push(
          protocol + '://' + tencentCdnOutput.host + ' (CNAME: ' + tencentCdnOutput.cname + '）'
        )
        this.state.hostName.push(inputs.hosts[i].host.replace('.', '_'))
      }
    }

    this.state.bucketName = inputs.bucketName
    this.state.region = inputs.region
    this.state.url = `${inputs.protocol}://${cosOriginAdd}`
    await this.save()

    const outputs = {
      url: this.state.url,
      env: inputs.env || {}
    }

    if (this.state.host) {
      outputs.host = this.state.host
    }

    this.context.debug(`Website deployed successfully to URL: ${this.state.url}.`)

    return outputs
  }

  /**
   * Remove
   */

  async remove() {
    this.context.status(`Removing`)

    this.context.debug(`Starting Website Removal.`)

    this.context.debug(`Removing Website bucket.`)
    const websiteBucket = await this.load('@serverless/tencent-cos', 'websiteBucket')
    await websiteBucket.remove()

    if (this.state.hostName && this.state.hostName.length > 0) {
      let tencentCdn
      for (let i = 0; i < this.state.hostName.length; i++) {
        tencentCdn = await this.load('@serverless/tencent-cdn', this.state.hostName[i])
        await tencentCdn.remove()
      }
    }

    this.state = {}
    await this.save()

    this.context.debug(`Finished Website Removal.`)
    return {}
  }
}

module.exports = Website
